import es
import playerlib
from helpers import *

# == ACHIEVEMENT CLASS ==================================
class Achievement:
    __parentID = None
    __id = None
    __name = None
    __description = None
    __totalProgress = None
    __playerProgress = {}
    
    def __init__(self, newParentID, newID, newName, newDescription, newTotalProgress = 1):
        self.__parentID = newParentID
        self.__id = newID
        self.__name = newName
        self.__description = newDescription
        self.__totalProgress = newTotalProgress
        self.__playerProgress = {}

    def __getattr__(self, name):
        if name == "parentid":
            return self.__parentID
        elif name == "id":
            return self.__id
        elif name == "name":
            return self.__name
        elif name == "description":
            return self.__description
        elif name == "totalprogress":
            return int(self.__totalProgress)
        elif name in self.__dict__.keys():
            return self.__dict__[name]
        else:
            raise AttributeError, name
    
    def __setattr__(self, name, value):
        if name == "id":
            self.__id = value
        elif name == "name":
            self.__name = value
        elif name == "description":
            self.__description = value
        elif name == "totalprogress":
            self.__totalProgress = int(value)
        else:
            self.__dict__[name] = value

    def stepProgressForPlayer(self, playerID, stepAmount = 1):
        backwardStep = False
        steamID = getUniqueSteam(playerID)
        
        progress = None
        if steamID in self.__playerProgress.keys():
            progress = int(self.__playerProgress[steamID])
            
        if not progress == None: 
            if progress == self.__totalProgress:
                if stepAmount >= 0: 
                    return None
                else:
                    backwardStep = True
            progress = progress + stepAmount
        else:
            progress = stepAmount
        
        if progress < 0:
            progress = 0
        if progress >= self.__totalProgress:
            progress = self.__totalProgress
            self.__doAchievementPlayer(playerID)
        elif backwardStep:
            self.__undoAchievementPlayer(playerID)
            
        self.__playerProgress[steamID] = progress
    
    def stepProgressForList(self, listFilter, stepAmount = 1):
        for ply in playerlib.getUseridList(listFilter):
            self.stepProgressForPlayer(ply, stepAmount)
    
    def stepProgressForAllActive(self, stepAmount = 1):
        for ply in es.getUseridList():
            self.stepProgressForPlayer(ply, stepAmount)
            
    def completeProgressForPlayer(self, playerID):
        steamID = getUniqueSteam(playerID)

        progress = None
        if steamID in self.__playerProgress.keys():
            progress = int(self.__playerProgress[steamID])

        if progress == None or progress < self.__totalProgress:
            self.__playerProgress[steamID] = self.__totalProgress
            self.__doAchievementPlayer(playerID)
    
    def completeProgressForList(self, listFilter):
        for ply in playerlib.getUseridList(listFilter):
            self.completeProgressForPlayer(ply)
                
    def completeProgressForAllActive(self):
        for ply in es.getUseridList():
            self.completeProgressForPlayer(ply)
                
    def resetProgressForPlayer(self, playerID):
        steamID = getUniqueSteam(playerID)

        progress = None
        if steamID in self.__playerProgress.keys():
            progress = int(self.__playerProgress[steamID])
            
        self.__playerProgress[steamID] = 0
        if not progress == None and progress == self.__totalProgress:
            self.__resetAchievementPlayer(playerID)
    
    def resetProgressForList(self, listFilter):
        for ply in playerlib.getUseridList(listFilter):
            self.resetProgressForPlayer(ply, stepAmount)
                
    def resetProgressForAllActive(self):
        for ply in es.getUseridList():
            self.resetProgressForPlayer(ply, stepAmount)
                                          
    def getPlayerProgress(self, playerID):
        steamID = getUniqueSteam(playerID)
        if "STEAM" in playerID:
            steamID = playerID
            
        if steamID in self.__playerProgress.keys():
            return int(self.__playerProgress[steamID])
        return 0
    
    def isPlayerCompleted(self, playerID):
        return self.getPlayerProgress(playerID) == self.__totalProgress
            
    def __doAchievementPlayer(self, playerID):
        self.__triggerEvent("achievement_achieved", playerID)     
    
    def __undoAchievementPlayer(self, playerID):
        self.__triggerEvent("achievement_unachieved", playerID)

    def __resetAchievementPlayer(self, playerID):
        self.__triggerEvent("achievement_reset", playerID)
    
    def __triggerEvent(self, eventName, playerID):
        es.event("initialize", eventName)
        es.event("setstring", eventName, "achievementgroupid", self.__parentID)
        es.event("setstring", eventName, "achievementid", self.__id)
        es.event("setstring", eventName, "playerid", playerID)
        es.event("fire", eventName)
# =======================================================
