import es
import cfglib
import popuplib
import cPickle
import os.path
from helpers import *
from AchievementGroup import AchievementGroup
from Achievement import Achievement
from odict import odict

# == ADDON INFO =========================================
info = es.AddonInfo()
info.name        = "Make-Your-Own Achievements"
info.version     = "v2.0.0"
info.author      = "The Bash"
info.url         = "http://addons.eventscripts.com/addons/view/myo_achievement"
info.basename    = "myo_achievement"
info.description = "Provides an easy way to add your own achievements by EventScripts."
info.cfgpath     = "config.cfg"
# =======================================================

# == CONFIG FILE CONFIGURATION ==========================
config = cfglib.AddonCFG(es.getAddonPath("myo_achievement") + "/" + info.cfgpath)

config.text("*****************************************") 
config.text("    " + info.name + " " + info.version)
config.text("    Configuration File")
config.text("*****************************************")
config.text("")
config.text("*****************************************")
config.text("    Required variables")
config.text("*****************************************")
 
frontOfMessage        = config.cvar("myoa_front", "[Achievements]", "This is used at the beginning of each message from myo_achievements.")
popupWidth            = config.cvar("myoa_popup_width", 30, "This is the maximum width in characters for each line in the popup.")
sayCommand            = config.cvar("myoa_popup_command", "!achievements", "This is the command to bring up the popups.")
otherPlayerSayCommand = config.cvar("myoa_others_popup_command", "!otherachievers", "This is the command to bring up the popups to get achievement details of other players.")
top10SayCommand       = config.cvar("myoa_top10_popup_command", "!top10achievers", "This is the command to show the top 10 achievers popup.")

config.write()
# =======================================================

# == CONSOLE COMMAND REGISTRATION =======================
es.regcmd("myoa_add_group", "myo_achievement/addGroupByConsole", "Command for adding achivement groups by console.")
es.regcmd("myoa_add_achievement", "myo_achievement/addAchievementByConsole", "Command for adding achievements by console.")
es.regcmd("myoa_step_progress_player", "myo_achievement/stepProgressPlayerByConsole", "Command for stepping progress by console.")
es.regcmd("myoa_step_progress_list", "myo_achievement/stepProgressListByConsole", "Command for stepping progress by console.")
es.regcmd("myoa_step_progress_all", "myo_achievement/stepProgressAllByConsole", "Command for stepping progress by console.")
es.regcmd("myoa_complete_progress_player", "myo_achievement/completeAchievementPlayerByConsole", "Command for completing progress by console.")
es.regcmd("myoa_complete_progress_list", "myo_achievement/completeAchievementListByConsole", "Command for completing progress by console.")
es.regcmd("myoa_complete_progress_all", "myo_achievement/completeAchievementAllByConsole", "Command for completing progress by console.")
es.regcmd("myoa_reset_progress_player", "myo_achievement/resetAchievementPlayerByConsole", "Command for resetting progress by console.")
es.regcmd("myoa_reset_progress_list", "myo_achievement/resetAchievementListByConsole", "Command for resetting progress by console.")
es.regcmd("myoa_reset_progress_all", "myo_achievement/resetAchievementAllByConsole", "Command for resetting progress by console.")
es.regcmd("myoa_get_player_progress", "myo_achievement/getPlayerProgressByConsole", "Command for returning the player progress by console.")
es.regcmd("myoa_is_player_completed", "myo_achievement/isPlayerCompletedByConsole", "Command for returning if the player has completed by console.")
# =======================================================

# == PRIVATE GLOBAL VARIABLES ===========================
_achievementGroups = odict()
_rankings = {}
_popupValues = {}
_popupUsers = {}
_popupQueries = {}
# =======================================================

# == ES EVENTS ==========================================
def load():
    global _achievementGroups
    global _rankings

    es.set("myo_achievement_version",info.version)
    es.makepublic("myo_achievement_version")
    es.loadevents("declare", es.getAddonPath("myo_achievement") + "/achievementevents.res")
    
    config.execute()

    if not es.exists("saycommand", sayCommand):
        es.regsaycmd(sayCommand, "myo_achievement/getAchievementGroups", "Shows the active achievements.")
    
    if not es.exists("saycommand", otherPlayerSayCommand):
        es.regsaycmd(otherPlayerSayCommand, "myo_achievement/getOtherPlayerAchievementGroups", "Shows the active achievements for other players.")
        
    if not es.exists("saycommand", top10SayCommand):
        es.regsaycmd(top10SayCommand, "myo_achievement/getTop10Players", "Shows the top 10 achievers.")
    
    addonPath = es.getAddonPath("myo_achievement")
    if os.path.exists(addonPath + "/database.db"):
        pickleFile = open(addonPath + "/database.db", "r")
        _achievementGroups = cPickle.load(pickleFile)
        pickleFile.close()
    
    if os.path.exists(addonPath + "/playerpairs.db"):
        pickleFile = open(addonPath + "/playerpairs.db", "r")
        playerPairs = cPickle.load(pickleFile)
        setPlayerPairs(playerPairs)
        pickleFile.close()
        
    if os.path.exists(addonPath + "/playerranks.db"):
        pickleFile = open(addonPath + "/playerranks.db", "r")
        _rankings = cPickle.load(pickleFile)
        pickleFile.close()
        
def es_map_start(ev):
    es.loadevents(es.getAddonPath("myo_achievement") + "/achievementevents.res")    
    saveToPickle()

def unload():
    saveToPickle()

def saveToPickle():
    global _achievementGroups
    global _rankings
    
    addonPath = es.getAddonPath("myo_achievement")

    pickleFile = open(addonPath + "/database.db", "w")
    cPickle.dump(_achievementGroups, pickleFile)
    pickleFile.close()
    
    pickleFile = open(addonPath + "/playerpairs.db", "w")
    playerPairs = getPlayerPairs()
    cPickle.dump(playerPairs, pickleFile)
    pickleFile.close() 
    
    pickleFile = open(addonPath + "/playerranks.db", "w")
    cPickle.dump(_rankings, pickleFile)
    pickleFile.close() 
# =======================================================

# == ACHIEVEMENT EVENTS =================================
def achievement_achieved(ev):
    global _rankings
    
    steamID = getUniqueSteam(ev["playerid"])
    if not steamID in _rankings.keys():
        _rankings[steamID] = 1
    else:
        _rankings[steamID] = int(_rankings[steamID]) + 1

    achievementName = _achievementGroups[ev["achievementgroupid"]][ev["achievementid"]].name
    es.msg("#multi", "#lightgreen%s#default %s" % (frontOfMessage, getMessage(achievementName, "achieved by", ev["playerid"])))
    
def achievement_unachieved(ev):
    global _rankings

    steamID = getUniqueSteam(ev["playerid"])
    if not steamID in _rankings.keys():
        _rankings[steamID] = 0
    else:
        _rankings[steamID] = int(_rankings[steamID]) - 1
        
    achievementName = _achievementGroups[ev["achievementgroupid"]][ev["achievementid"]].name
    es.msg("#multi", "#lightgreen%s#default %s" % (frontOfMessage, getMessage(achievementName, "un-achieved by", ev["playerid"])))
    
def achievement_reset(ev):
    global _rankings

    steamID = getUniqueSteam(ev["playerid"])
    if not steamID in _rankings.keys():
        _rankings[steamID] = 0
    else:
        _rankings[steamID] = int(_rankings[steamID]) - 1
        
    achievementName = _achievementGroups[ev["achievementgroupid"]][ev["achievementid"]].name
    es.msg("#multi", "#lightgreen%s#default %s" % (frontOfMessage, getMessage(achievementName, "reset for", ev["playerid"])))
# =======================================================

# == ADDITIONAL FUNCTIONS ===============================
def AddAchievementGroup(newID, newName, newDescription):
    global _achievementGroups
    if not newID in _achievementGroups.keys():
        _achievementGroups[newID] = AchievementGroup(newID, newName, newDescription)
    return _achievementGroups[newID]
# =======================================================

# == ACHIEVEMENT POPUPS =================================
def getAchievementGroups():
    global _popupQueries
    if es.getcmduserid() in _popupQueries.keys():
        del _popupQueries[es.getcmduserid()]
        
    getPlayerAchievementGroups(getUniqueSteam(es.getcmduserid()), es.getcmduserid())

def getOtherPlayerAchievementGroups(query = None, userid = None):
    global _popupQueries
    
    if userid == None:
        userid = es.getcmduserid()
    
    results = []
    if query == None:
        if es.getargc() == 1:
            for ply in es.getUseridList():
                results.append(getUniqueSteam(ply))
            _popupQueries[userid] = ""
        else:
            query = ""
            for i in range(1, es.getargc()):
                if not i == 1:
                    query = query + " "
                query = query + es.getargv(1).lower()           
            players = getPlayerPairs()
            for steamID in players.keys():
                if query in players[steamID].lower():
                    results.append(steamID)
            _popupQueries[userid] = query
    else:
        if query == "":
            for ply in es.getUseridList():
                results.append(getUniqueSteam(ply))
            _popupQueries[userid] = ""
        else:        
            players = getPlayerPairs()
            for steamID in players.keys():
                if query in players[steamID].lower():
                    results.append(steamID)        
            _popupQueries[userid] = query
                
    showPlayerList(results, userid)

def showPlayerList(results, sendToPlayerID):
    anyResults = False
    for result in results:
        anyResults = True
        break
    
    if anyResults:
        myPopup = popuplib.easymenu("achievementPlayersMenu", None, getPlayerPopup)
        myPopup.settitle("Choose a player")
        
        for result in results:
            myPopup.addoption(result, getPlayerName(result))
    
        myPopup.send(sendToPlayerID)
        myPopup.delete()
    else:
        myPopup = popuplib.create("achievementPlayersMenu")    
        myPopup.addline("No players found with the search query:")
        myPopup.addline(_popupQueries[sendToPlayerID])        
        myPopup.send(es.getcmduserid())
        myPopup.delete()
    
def getPlayerPopup(userid, choice, popupname):
    getPlayerAchievementGroups(choice, userid)
        
def getPlayerAchievementGroups(playerSteam, sendToPlayerID):
    global _achievementGroups
        
    myPopup = popuplib.easymenu("achievementGroupsMenu", None, getAchievements)
    myPopup.settitle("Achievement Groups - %s" % getPlayerName(playerSteam))
    
    for achievementGroup in _achievementGroups.values():
        achievementCount = 0
        totalAchievementCount = 0
        for achievement in achievementGroup.achievements.values():
            totalAchievementCount = totalAchievementCount + 1
            if achievement.isPlayerCompleted(playerSteam):
                achievementCount = achievementCount + 1
        myPopup.addoption((playerSteam, achievementGroup), "%s (%d/%d)" % (achievementGroup.name, achievementCount, totalAchievementCount))
    
    if sendToPlayerID in _popupQueries.keys():
        myPopup.menuselect = getPlayerAchievementGroupsEx
    myPopup.send(sendToPlayerID)
    myPopup.delete() 

def getPlayerAchievementGroupsEx(userid, choice, popupname):
    if type(choice) is not tuple:
        if int(choice) == 10:
            getOtherPlayerAchievementGroups(_popupQueries[userid], userid)

def getAchievements(userid, choice, popupname):
    global _popupUsers
    
    myPopup = popuplib.easymenu("achievementsMenu", None, getAchievement)
    myPopup.settitle("Achievements [%s] - %s" % (choice[1].name, getPlayerName(choice[0])))
    for achievement in choice[1].achievements.values():
        playerProgress = achievement.getPlayerProgress(choice[0])
        if playerProgress == None:
            playerProgress = "0"
            
        myPopup.addoption((choice[0], achievement), "%s (%d/%d)" % (achievement.name, achievement.getPlayerProgress(choice[0]), achievement.totalprogress))
    
    _popupUsers[userid] = choice[0]
    myPopup.menuselect = getAchievementEx
    myPopup.send(userid)
    myPopup.delete()

def getAchievementEx(userid, choice, popupname):
    if type(choice) is not tuple:
        if int(choice) == 10:
            getPlayerAchievementGroups(_popupUsers[userid], userid)

def getAchievement(userid, choice, popupname):
    global _achievementGroups
    global _popupValues
    global _popupUsers
        
    myPopup = popuplib.create("achievementMenu")    
    myPopup.addline(choice[1].name + " Achievement")
    myPopup.addline(" ")
    
    descriptionlist = choice[1].description.split(" ")
    concat = ""
    length = 0
    for word in descriptionlist:
        length = length + len(word) + 1
        if length > int(popupWidth):
            myPopup.addline(concat)
            concat = ""
            length = len(word) + 1
        concat = concat + word + " "

    myPopup.addline(concat)
    myPopup.addline(" ")        
    
    playerProgress = choice[1].getPlayerProgress(choice[0])
    if playerProgress == None:
        playerProgress = "0"
    
    myPopup.addline("Progress for %s: %s/%s" % (getPlayerName(choice[0]), playerProgress, choice[1].totalprogress))
    myPopup.addline(" ")
    myPopup.addline("0. Go Back")
    
    _popupUsers[userid] = choice[0]
    _popupValues[userid] = _achievementGroups[choice[1].parentid]
    myPopup.menuselect = getAchievementOutEx
    myPopup.send(userid)
    myPopup.delete()

def getAchievementOutEx(userid, choice, popupname):
    if int(choice) == 10:
        getAchievements(userid, (_popupUsers[userid], _popupValues[userid]), popupname)
        
def getTop10Players():
    global _rankings
    myPopup = popuplib.create("top10AchievementMenu")    
    myPopup.addline("Top 10 Achievers")
    myPopup.addline(" ")
    
    orderedList = sortDictByValue(_rankings)
    
    position = 1
    for steamID in orderedList:
        if int(_rankings[steamID]) == 1:
            myPopup.addline("%s. %s (1 achievement)" % (str(position), getPlayerName(steamID)))
        else:
            myPopup.addline("%s. %s (%s achievements)" % (str(position), getPlayerName(steamID), _rankings[steamID]))
        position = position + 1
        if position == 11:
            break
    
    if position == 1:
        myPopup.addline("<<No Entries>>")
    
    myPopup.send(es.getcmduserid())
    myPopup.delete()
# =======================================================

# == CONSOLE COMMANDS ===================================
def addGroupByConsole():
    if es.getargc() == 4:
        AddAchievementGroup(es.getargv(1), es.getargv(2), es.getargv(3))
    else:
        _showSyntaxErrorMessage("myoa_add_group", "myoa_add_group groupid groupname groupdescription")

def addAchievementByConsole():
    global _achievementGroups
    if es.getargc() == 5:
        _achievementGroups[es.getargv(1)][es.getargv(2)] = (es.getargv(3), es.getargv(4))
    elif es.getargc() == 6:
        _achievementGroups[es.getargv(1)][es.getargv(2)] = (es.getargv(3), es.getargv(4), int(es.getargv(5)))
    else:
        _showSyntaxErrorMessage("myoa_add_achievement", "myoa_add_achievement groupid achievementid achievementname achievementdescription [totalprogress = 1]")

def stepProgressPlayerByConsole():
    global _achievementGroups
    if es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].stepProgressForPlayer(es.getargv(3))    
    elif es.getargc() == 5:
        _achievementGroups[es.getargv(1)][es.getargv(2)].stepProgressForPlayer(es.getargv(3), es.getargv(4))
    else:
        _showSyntaxErrorMessage("myoa_step_progress_player", "myoa_step_progress_player groupid achievementid playerid [stepamount = 1]")

def stepProgressListByConsole():
    global _achievementGroups
    if es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].stepProgressForList(es.getargv(3))    
    elif es.getargc() == 5:
        _achievementGroups[es.getargv(1)][es.getargv(2)].stepProgressForList(es.getargv(3), es.getargv(4))
    else:
        _showSyntaxErrorMessage("myoa_step_progress_list", "myoa_step_progress_list groupid achievementid listfilter [stepamount = 1]")
        
def stepProgressAllByConsole():
    global _achievementGroups
    if es.getargc() == 3:
        _achievementGroups[es.getargv(1)][es.getargv(2)].stepProgressForAllActive()    
    elif es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].stepProgressForAllActive(es.getargv(3))
    else:
        _showSyntaxErrorMessage("myoa_step_progress_all", "myoa_step_progress_all groupid achievementid [stepamount = 1]")
        
def completeProgressPlayerByConsole():
    global _achievementGroups
    if es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].completeProgressForPlayer(es.getargv(3))    
    elif es.getargc() == 5:
        _achievementGroups[es.getargv(1)][es.getargv(2)].completeProgressForPlayer(es.getargv(3), es.getargv(4))
    else:
        _showSyntaxErrorMessage("myoa_complete_progress_player", "myoa_complete_progress_player groupid achievementid playerid")

def completeProgressListByConsole():
    global _achievementGroups
    if es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].completeProgressForList(es.getargv(3))    
    elif es.getargc() == 5:
        _achievementGroups[es.getargv(1)][es.getargv(2)].completeProgressForList(es.getargv(3), es.getargv(4))
    else:
        _showSyntaxErrorMessage("myoa_complete_progress_list", "myoa_complete_progress_list groupid achievementid listfilter")
        
def completeProgressAllByConsole():
    global _achievementGroups
    if es.getargc() == 3:
        _achievementGroups[es.getargv(1)][es.getargv(2)].completeProgressForAllActive()    
    elif es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].completeProgressForAllActive(es.getargv(3))
    else:
        _showSyntaxErrorMessage("myoa_complete_progress_all", "myoa_complete_progress_all groupid achievementid")

def resetProgressPlayerByConsole():
    global _achievementGroups
    if es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].resetProgressForPlayer(es.getargv(3))    
    elif es.getargc() == 5:
        _achievementGroups[es.getargv(1)][es.getargv(2)].resetProgressForPlayer(es.getargv(3), es.getargv(4))
    else:
        _showSyntaxErrorMessage("myoa_reset_progress_player", "myoa_reset_progress_player groupid achievementid playerid")

def resetProgressListByConsole():
    global _achievementGroups
    if es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].resetProgressForList(es.getargv(3))    
    elif es.getargc() == 5:
        _achievementGroups[es.getargv(1)][es.getargv(2)].resetProgressForList(es.getargv(3), es.getargv(4))
    else:
        _showSyntaxErrorMessage("myoa_reset_progress_list", "myoa_reset_progress_list groupid achievementid listfilter")
        
def resetProgressAllByConsole():
    global _achievementGroups
    if es.getargc() == 3:
        _achievementGroups[es.getargv(1)][es.getargv(2)].resetProgressForAllActive()    
    elif es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].resetProgressForAllActive(es.getargv(3))
    else:
        _showSyntaxErrorMessage("myoa_reset_progress_all", "myoa_reset_progress_all groupid achievementid")

def getPlayerProgressByConsole():
    global _achievementGroups
    if es.getargc() == 4:
        es.set("myoa_player_progress", _achievementGroups[es.getargv(1)][es.getargv(2)].getPlayerProgress(es.getargv(3)))    
    else:
        _showSyntaxErrorMessage("myoa_get_player_progress", "myoa_get_player_progress groupid achievementid playerid")

def isPlayerCompletedByConsole():
    global _achievementGroups
    if es.getargc() == 4:
        if _achievementGroups[es.getargv(1)][es.getargv(2)].isPlayerCompleted(es.getargv(3)) == True:
            es.set("myoa_player_complete", 1)
        else:
            es.set("myoa_player_complete", 0)
    else:
        _showSyntaxErrorMessage("myoa_is_player_completed", "myoa_is_player_completed groupid achievementid playerid")

def _showSyntaxErrorMessage(command, requiredArgs):
    es.dbgmsg(0, "Invalid syntax for %s (wrong number of arguments).\nRequired arguments: %s" % (command, requiredArgs))

# =======================================================